import { QueryBuilderState } from '../store/querybuilderSlice';
import { URLParams, setParam } from '../api/url';
import { Broker } from './broker';
import { VisState } from '../store/visualizationSlice';

// export function wsGetState() {
//   Broker.instance().subscribe((data) => dispatch(readInSchemaFromBackend(data)), 'schema_result');
//   Broker.instance().sendMessage({});
// }

export const databaseNameMapping: string[] = ['arangodb', 'neo4j'];
export const databaseProtocolMapping: string[] = ['neo4j://', 'neo4j+s://', 'bolt://', 'bolt+s://'];

export enum DatabaseType {
  ArangoDB = 0,
  Neo4j = 1,
}

export enum DatabaseStatus {
  untested = 0,
  testing = 1,
  tested = 2,
}

export type DatabaseInfo = {
  internalDatabaseName: string;
  url: string;
  port: number;
  protocol: string;
  username: string;
  password: string;
  type: number;
};

export const nilUUID = '00000000-0000-0000-0000-000000000000';

export type SaveStateI = {
  id: string;
  user_id: string;
  name: string;
  db: DatabaseInfo;
  schema: any;
  queryBuilder: QueryBuilderState;
  visualization: any;
  share_state: any;
};

export function wsGetState(saveStateId: string) {
  Broker.instance().sendMessage({
    key: 'state',
    subKey: 'get',
    body: { saveStateId: saveStateId }, //messageTypeGetSaveState
  });
}

export function wsGetStates() {
  Broker.instance().sendMessage({
    key: 'state',
    subKey: 'getAll',
  });
}

export function wsSelectState(saveStateId: string | undefined) {
  if (saveStateId === undefined) saveStateId = '';
  Broker.instance().sendMessage({
    key: 'state',
    subKey: 'select',
    body: { saveStateId: saveStateId }, //messageTypeGetSaveState
  });

  Broker.instance().useSaveStateID(saveStateId);
  setParam(URLParams.saveState, saveStateId);
}

export function wsCreateState(request: SaveStateI) {
  Broker.instance().sendMessage({
    key: 'state',
    subKey: 'create',
    body: request, //SaveStateModel
  });
}

export function wsDeleteState(id: string) {
  Broker.instance().sendMessage({
    key: 'state',
    subKey: 'delete',
    body: { saveStateId: id }, //messageTypeGetSaveState
  });
}

export function wsTestSaveStateConnection(id: string) {
  Broker.instance().sendMessage({
    key: 'state',
    subKey: 'testConnection',
    body: { saveStateId: id }, //messageTypeGetSaveState
  });
}

export function wsUpdateState(request: SaveStateI) {
  Broker.instance().sendMessage({
    key: 'state',
    subKey: 'update',
    body: request, //SaveStateModel
  });
}

export type TestDatabaseConnectionResponse = { status: 'success' | 'fail'; saveStateID: string };
export function wsTestDatabaseConnection(dbConnection: DatabaseInfo) {
  Broker.instance().sendMessage({
    key: 'dbConnection',
    subKey: 'testConnection',
    body: dbConnection, //DBConnectionModel
  });
}
